//****************************************************************************
//
// Copyright (c) ALTAP, spol. s r.o. All rights reserved.
//
// This is a part of the Altap Salamander SDK library.
//
// The SDK is provided "AS IS" and without warranty of any kind and 
// ALTAP EXPRESSLY DISCLAIMS ALL WARRANTIES, EXPRESS AND IMPLIED, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE and NON-INFRINGEMENT.
//
//****************************************************************************

#include <windows.h>
#include <commctrl.h>
#ifdef _MSC_VER
#include <crtdbg.h>
#endif // _MSC_VER

#ifdef TRACE_ENABLE

#include <ostream>
#include <streambuf>

#if defined(_DEBUG) && defined(_MSC_VER)  // without passing file+line to 'new' operator, list of memory leaks shows only 'crtdbg.h(552)'
#define new new(_NORMAL_BLOCK, __FILE__, __LINE__)
#endif

#include "spl_base.h"
#include "dbg.h"

#pragma warning (disable: 4074)
#pragma init_seg(compiler)

C__Trace __Trace;

//*****************************************************************************
//
// C__Trace
//

C__Trace::C__Trace() : TraceStrStream(&TraceStringBuf)
{
  InitializeCriticalSection(&CriticalSection);
}

C__Trace::~C__Trace()
{
  DeleteCriticalSection(&CriticalSection);
}

C__Trace &
C__Trace::SetInfo(const char *file, int line)
{
  File = file;
  Line = line;
  return *this;
}

void
C__Trace::SendMessageToServer(BOOL information)
{
  TraceStrStream.flush();             // flushnuti do bufferu
  if (SalamanderDebug != NULL)
  {
    if (information) SalamanderDebug->TraceI(File, Line, TraceStringBuf.c_str());
    else SalamanderDebug->TraceE(File, Line, TraceStringBuf.c_str());
  }
  TraceStringBuf.erase();       // priprava pro dalsi trace
  LeaveCriticalSection(&CriticalSection);
}

//*****************************************************************************

#ifdef _DEBUG
#undef memcpy
void *_sal_safe_memcpy(void *dest, const void *src, size_t count)
{
  if ((char *)dest + count > src && (char *)src + count > dest)
  {
    TRACE_E("_sal_safe_memcpy: source and destination of memcpy overlap!");
  }
  return memcpy(dest, src, count);
}
#endif // _DEBUG

#endif // TRACE_ENABLE
