//****************************************************************************
//
// Copyright (c) ALTAP, spol. s r.o. All rights reserved.
//
// This is a part of the Altap Salamander SDK library.
//
// The SDK is provided "AS IS" and without warranty of any kind and 
// ALTAP EXPRESSLY DISCLAIMS ALL WARRANTIES, EXPRESS AND IMPLIED, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE and NON-INFRINGEMENT.
//
//****************************************************************************

#ifndef __SPL_VERS_H
#define __SPL_VERS_H

// conversion macros num->str
#define VERSINFO_xstr(s) VERSINFO_str(s)
#define VERSINFO_str(s) #s

#define VERSINFO_SALAMANDER_MAJOR       2
#define VERSINFO_SALAMANDER_MINORA      5
#define VERSINFO_SALAMANDER_MINORB      4

#if (VERSINFO_SALAMANDER_MINORB == 0)  // nulu na setinach nepiseme 2.50 -> 2.5
#define VERSINFO_SALAMANDER_VERSION     VERSINFO_xstr(VERSINFO_SALAMANDER_MAJOR) "." VERSINFO_xstr(VERSINFO_SALAMANDER_MINORA) VERSINFO_BETAVERSION_TXT
#else
#define VERSINFO_SALAMANDER_VERSION     VERSINFO_xstr(VERSINFO_SALAMANDER_MAJOR) "." VERSINFO_xstr(VERSINFO_SALAMANDER_MINORA) VERSINFO_xstr(VERSINFO_SALAMANDER_MINORB) VERSINFO_BETAVERSION_TXT
#endif

#ifdef VERSINFO_MAJOR  // je definovane jen pokud se pouziva z pluginu
#if (VERSINFO_MINORB == 0)  // nulu na setinach nepiseme 2.50 -> 2.5
#define VERSINFO_VERSION     VERSINFO_xstr(VERSINFO_MAJOR) "." VERSINFO_xstr(VERSINFO_MINORA) VERSINFO_BETAVERSION_TXT
#else
#define VERSINFO_VERSION     VERSINFO_xstr(VERSINFO_MAJOR) "." VERSINFO_xstr(VERSINFO_MINORA) VERSINFO_xstr(VERSINFO_MINORB) VERSINFO_BETAVERSION_TXT
#endif
#endif


// VERSINFO_BUILDNUMBER:
//
// Slouzi ke snadnemu odliseni verzi vsech modulu mezi jednotlivymi verzemi
// Salamandera (jde o posledni komponentu cisla verze vsech pluginu a
// Salamandera). Zvysovat s kazdou verzi (IB, DB, PB, beta, release nebo i
// jen testovaci verze poslana jednomu uzivateli). Prehled ruznych typu verzi
// je v souboru doc\versions.txt. Vzdy zavest komentar s popisem, ke ktere
// verzi Salamandera patri nove pouzite cislo buildu.
//
// Prehled pouzitych hodnot VERSINFO_BUILDNUMBER:
// 9 - 2.5 beta 9
// 10 - 2.5 beta 10
// 11 - 2.5 beta 11
// 13 - 2.5 RC1
// 14 - 2.5 RC2
// 15 - 2.5 RC3
// 0 - 2.5
// 16 - 2.51
// 18 - 2.52 beta 1
// 29 - 2.52 beta 2
// 32 - 2.52
// 49 - 2.53 beta 1
// 57 - 2.53 beta 2
// 63 - 2.53
// 64 - 2.53 (SDK)
// 65 - 2.53 (CB 65)
// 66 - 2.54 beta 1 (DB 66)
// 67 - 2.54 beta 1 (CB 67)
// 68 - 2.54 beta 1 (IB 68)
// 69 - 2.54
// 70 - 2.54 (SDK)
// 71 - 2.54 (CB 71)

#define VERSINFO_BUILDNUMBER     71


// VERSINFO_BETAVERSION_TXT:
//
// Meni se s kazdym buildem, v pripade release verze bude VERSINFO_BETAVERSION_TXT="".
// Pokud vydavame specialni opravne beta verze typu 2.5 beta 9a, zvysime
// VERSINFO_BUILDNUMBER o jedna a dame VERSINFO_BETAVERSION_TXT==" beta 9a".

#define VERSINFO_BETAVERSION_TXT " (CB 71)"  // priklady: "" (pro release verze), " beta 2", " beta 2 (SDK)", " RC1", " beta 2 (IB 21)", " beta 2 (DB 21)", " beta 2 (PB 21)"


// LAST_VERSION_OF_SALAMANDER:
//
// Podpora pro kontrolu aktualnosti verze Salamandera, kterou interni pluginy
// (distribuovane v jednom balicku se Salamanderem) provadi behem entry-pointu
// (SalamanderPluginEntry) viz metoda CSalamanderPluginEntryAbstract::GetVersion()
// (v spl_base.h). Slouzi hlavne pro jednoduchost: interni plugin muze volat
// jakoukoliv metodu z rozhrani Salamandera, protoze po kontrole na posledni
// verzi Salamandera ma jistotu, ze ji Salamander obsahuje (hrozi mu jen load
// do novejsi verze Salamandera, ktery tyto metody musi tez obsahovat).
//
// Pouziva se i opacne: aby mel interni plugin jistotu, ze mu Salamander bude
// volat vsechny metody (vcetne nejnovejsich), vraci tuto verzi, jako verzi,
// pro kterou byl plugin postaven (viz export pluginu SalamanderPluginGetReqVer).
//
// Pokud nektery plugin vraci z SalamanderPluginGetReqVer nizsi verzi nez
// LAST_VERSION_OF_SALAMANDER (pro zpetnou kompatibilitu se starsimi verzemi
// Salamandera), mel by pridat export SalamanderPluginGetSDKVer a vracet z nej
// LAST_VERSION_OF_SALAMANDER (verze SDK pouzita pro stavbu pluginu), aby mohl
// Salamander (napr. aktualni nebo novejsi) pouzivat i metody pluginu, ktere
// ve verzi vracene z SalamanderPluginGetReqVer jeste nebyly.
//
// Pri zmenach v rozhrani je potreba dodrzet postup uvedeny v doc\how_to_change.txt.
//
// Prehled pouzitych hodnot LAST_VERSION_OF_SALAMANDER:
//   1  - 1.6 beta 4 + 5
//   2  - 1.6 beta 6
//   3  - 1.6 beta 7
//   4  - 2.0
//   5  - 2.5 beta 1
//   6  - 2.5 beta 2
//   7  - 2.5 beta 3
//   8  - 2.5 beta 4
//   9  - 2.5 beta 5
//   10 - 2.5 beta 6
//   11 - 2.5 beta 7
//   12 - 2.5 beta 8
//   13 - 2.5 beta 9
//   14 - 2.5 beta 10
//   15 - 2.5 beta 10a
//   16 - 2.5 beta 11
//   17 - 2.5 beta 12 (jen interni, pustili jsme misto ni RC1)
//   18 - 2.5 RC1
//   19 - 2.5 RC2
//   20 - 2.5 RC3
//   21 - 2.5
//   22 - 2.51
//   23 - 2.52 beta 1 (POZOR: nekompatibilni SDK s predchozimi a dalsimi verzemi)
//   29 - 2.52 beta 2
//   31 - 2.52
//   39 - 2.53 beta 1 + 2.53 beta 1a
//   41 - 2.53 beta 2
//   43 - 2.53
//   44 - 2.54 beta 1 (DB 66)
//   45 - 2.54

#define LAST_VERSION_OF_SALAMANDER         45
#define REQUIRE_LAST_VERSION_OF_SALAMANDER "This plugin requires Altap Salamander 2.54 or later."

#endif // __SPL_VERS_H
